@@ -1,18 +1,19 @@
 package TAP::Parser::SourceHandler::Feature;
-$TAP::Parser::SourceHandler::Feature::VERSION = '0.56';
+$TAP::Parser::SourceHandler::Feature::VERSION = '0.67';
 use strict;
 use warnings;
 
 use Path::Class qw/file/;
+use Test2::API qw/context/;
 
 use base 'TAP::Parser::SourceHandler';
 
-use TAP::Parser::Iterator::Stream;
+use TAP::Parser::Iterator::PherkinStream;
 
 use App::pherkin;
 
 use Test::BDD::Cucumber::Loader;
-use Test::BDD::Cucumber::Harness::TestBuilder;
+use Test::BDD::Cucumber::Harness::TAP;
 
 use Path::Class qw/file/;
 
@@ -73,30 +74,26 @@ sub make_iterator {
     #  (pipes are stdio buffered by default)
     $output_fh->autoflush(1);
 
-    my $tb = Test::Builder->create();
-    $tb->output($output_fh);
-
-    my $pid = fork;
-    if ($pid) {
-        close $output_fh;
-        return TAP::Parser::Iterator::Stream->new($input_fh);
-    }
-
-    close $input_fh;
-    my $harness = Test::BDD::Cucumber::Harness::TestBuilder->new(
-        {   fail_skip    => 1,
-            _tb_instance => $tb,
-        }
-    );
-
     my $dir     = $source->meta->{'file'}->{'dir'};
     my $runtime = $source->{'pherkins'}->{$dir}
         || die "No pherkin instantiation for [$dir]";
 
     my $executor = $runtime->{'executor'};
     my $pherkin  = $runtime->{'pherkin'};
-    $pherkin->harness($harness);
 
+    my $pid = fork;
+    if ($pid) {
+        close $output_fh;
+        return TAP::Parser::Iterator::PherkinStream->new($input_fh, $pherkin, $pid);
+    }
+
+    close $input_fh;
+    my $harness = Test::BDD::Cucumber::Harness::TAP->new({ fail_skip => 1 });
+
+    my $context = context();
+    # Without the step to set the handles TAP will end up on STDOUT/STDERR
+    $context->hub->format->set_handles([$output_fh, $output_fh]);
+    $pherkin->harness($harness);
     my $filename = file( $dir . $source->meta->{'file'}->{'basename'} ) . '';
 
     my $feature = $runtime->{'features'}->{$filename}
@@ -104,6 +101,7 @@ sub make_iterator {
         . ( join '; ', keys %{ $runtime->{'features'} } );
 
     $pherkin->_run_tests( $executor, $feature );
+    $context->release;
 
     close $output_fh;
     exit;
