@@ -1,12 +1,12 @@
 package Test::BDD::Cucumber::Parser;
-$Test::BDD::Cucumber::Parser::VERSION = '0.57';
+$Test::BDD::Cucumber::Parser::VERSION = '0.67';
 =head1 NAME
 
 Test::BDD::Cucumber::Parser - Parse Feature files
 
 =head1 VERSION
 
-version 0.57
+version 0.67
 
 =head1 DESCRIPTION
 
@@ -32,6 +32,7 @@ L<Test::BDD::Cucumber::Model::Feature> object on success.
 use strict;
 use warnings;
 
+use Test::BDD::Cucumber::Model::Dataset;
 use Test::BDD::Cucumber::Model::Document;
 use Test::BDD::Cucumber::Model::Feature;
 use Test::BDD::Cucumber::Model::Scenario;
@@ -81,8 +82,11 @@ sub _construct {
 
     $feature->language( $class->_extract_language( \@lines ) );
 
-    my $self = { langdef => langdef( $feature->language ) };
-    bless $self, $class;
+    my $langdef = langdef( $feature->language );
+    my $self = bless {
+        langdef => $langdef,
+        _construct_matchers( $langdef )
+    }, $class;
 
     $self->_extract_scenarios(
         $self->_extract_conditions_of_satisfaction(
@@ -93,6 +97,67 @@ sub _construct {
     return $feature;
 }
 
+sub _construct_matchers {
+    my ($l) = @_;
+    my $step_line_kw_cont =
+        join('|', map { $l->{$_} } qw/given and when then but/);
+    my $step_line_kw_first =
+        join('|', map { $l->{$_} } qw/given when then/);
+    my $scenario_line_kw =
+        join('|', map { $l->{$_} } qw/background scenario scenarioOutline/);
+
+    return (
+        _step_line_first => qr/^($step_line_kw_first)(.+)/,
+        _step_line_cont  => qr/^($step_line_kw_cont)(.+)/,
+        _feature_line    => qr/^($l->{feature}): (.+)/,
+        _scenario_line   => qr/^($scenario_line_kw): ?(.*)?/,
+        _examples_line   => qr/^($l->{examples}): ?(.+)?$/,
+        _table_line      => qr/^\s*\|/,
+        _tags_line       => qr/\@([^\s]+)/,
+        );
+}
+
+sub _is_step_line {
+    my ($self, $continuation, $line) = @_;
+
+    if ($continuation) {
+        return $line =~ $self->{_step_line_cont};
+    }
+    else {
+        return $line =~ $self->{_step_line_first};
+    }
+}
+
+sub _is_feature_line {
+    my ($self, $line) = @_;
+
+    return $line =~ $self->{_feature_line};
+}
+
+sub _is_scenario_line {
+    my ($self, $line) = @_;
+
+    return $line =~ $self->{_scenario_line};
+}
+
+sub _is_table_line {
+    my ($self, $line) = @_;
+
+    return $line =~ $self->{_table_line};
+}
+
+sub _is_tags_line {
+    my ($self, $line) = @_;
+
+    return $line =~ $self->{_tags_line};
+}
+
+sub _is_examples_line {
+    my ($self, $line) = @_;
+
+    return $line =~ $self->{_examples_line};
+}
+
 sub _extract_language {
     my ( $self, $lines ) = @_;
 
@@ -122,8 +187,10 @@ sub _extract_feature_name {
         next if $line->is_comment;
         last if $line->is_blank;
 
-        if ( $line->content =~ m/^(?:$self->{langdef}->{feature}): (.+)/ ) {
-            $feature->name($1);
+        if ( my ($keyword, $name) =
+             $self->_is_feature_line( $line->content ) ) {
+            $feature->name($name);
+            $feature->keyword_original($keyword);
             $feature->name_line($line);
             $feature->tags( \@feature_tags );
 
@@ -153,10 +220,8 @@ sub _extract_conditions_of_satisfaction {
     while ( my $line = shift(@lines) ) {
         next if $line->is_comment || $line->is_blank;
 
-        my $langdef = $self->{langdef};
-        if ( $line->content =~
-            m/^((?:$langdef->{background}):|(?:$langdef->{scenario}):|@)/ )
-        {
+        if ( $self->_is_scenario_line( $line->content )
+             or $self->_is_tags_line( $line->content ) ) {
             unshift( @lines, $line );
             last;
         } else {
@@ -167,20 +232,64 @@ sub _extract_conditions_of_satisfaction {
     return $feature, $self->_remove_next_blanks(@lines);
 }
 
+sub _finish_scenario {
+    my ($self, $feature, $line) = @_;
+    # Catch Scenario outlines without examples
+    if ( @{ $feature->scenarios } ) {
+        my $last_scenario = $feature->scenarios->[-1];
+        if ( $last_scenario->keyword_original =~ m/^($self->{langdef}->{scenarioOutline})/
+             && !@{ $last_scenario->data } )
+        {
+            die parse_error_from_line(
+                "Outline scenario expects 'Examples:' section",
+                $line || $last_scenario->line );
+        }
+    }
+ }
+
 sub _extract_scenarios {
     my ( $self, $feature, @lines ) = @_;
     my $scenarios = 0;
-    my @scenario_tags;
+    my $langdef   = $self->{langdef};
+    my @tags;
 
     while ( my $line = shift(@lines) ) {
         next if $line->is_comment || $line->is_blank;
 
-        my $langdef = $self->{langdef};
-        if ( $line->content =~
-m/^((?:$langdef->{background})|(?:$langdef->{scenario})|(?:$langdef->{scenario_outline})): ?(.+)?/
-          )
-        {
-            my ( $type, $name ) = ( $1, $2 );
+        if ( my ( $type, $name ) =
+             $self->_is_examples_line( $line->content ) ) {
+
+            die q{'Examples:' line before scenario definition}
+                unless @{$feature->scenarios};
+
+            my $dataset = Test::BDD::Cucumber::Model::Dataset->new(
+                ( $name ? ( name => $name ) : () ),
+                tags => ( @tags ?
+                          [ @{ $feature->scenarios->[-1]->tags }, @tags ]
+                          # Reuse the ref to the scenario tags to allow
+                          # detecting 'no dataset tags' in ::Scenario
+                          : $feature->scenarios->[-1]->tags ),
+                line => $line,
+                );
+            @tags = ();
+            if (@{$feature->scenarios->[-1]->datasets}) {
+                my $prev_ds = $feature->scenarios->[-1]->datasets->[0];
+                my $prev_ds_cols = join '|', keys %{$prev_ds->data->[0]};
+                my $cur_ds_cols = join '|', keys %{$dataset->data->[0]};
+                die parse_error_from_line(
+                    q{'Examples:' not in line with previous 'Examples:'}, $line )
+                    if $prev_ds_cols ne $cur_ds_cols;
+            }
+            push @{$feature->scenarios->[-1]->datasets}, $dataset;
+
+            @lines = $self->_extract_examples_description( $dataset, @lines );
+            @lines = $self->_extract_table( 6, $dataset,
+                                            $self->_remove_next_blanks(@lines) );
+        }
+        elsif ( ( $type, $name ) =
+                $self->_is_scenario_line( $line->content ) ) {
+
+            $self->_finish_scenario( $feature, $line );
 
             # Only one background section, and it must be the first
             if ( $scenarios++ && $type =~ m/^($langdef->{background})/ ) {
@@ -192,24 +301,22 @@ m/^((?:$langdef->{background})|(?:$langdef->{scenario})|(?:$langdef->{scenario_o
             my $scenario = Test::BDD::Cucumber::Model::Scenario->new(
                 {
                     ( $name ? ( name => $name ) : () ),
-                    background => $type =~ m/^($langdef->{background})/ ? 1 : 0,
-                    line => $line,
-                    tags => [ @{ $feature->tags }, @scenario_tags ]
+                    background       => $type =~ m/^($langdef->{background})/ ? 1 : 0,
+                    keyword          =>
+                        ($type =~ m/^($langdef->{background})/ ? 'Background'
+                         : ($type =~ m/^($langdef->{scenarioOutline})/
+                            ? 'Scenario Outline' : 'Scenario')),
+                    keyword_original => $type,
+                    line             => $line,
+                    tags             => [ @{ $feature->tags }, @tags ]
                 }
             );
-            @scenario_tags = ();
+            @tags = ();
 
             # Attempt to populate it
+            @lines = $self->_extract_scenario_description($scenario, @lines);
             @lines = $self->_extract_steps( $feature, $scenario, @lines );
 
-            # Catch Scenario outlines without examples
-            if ( $type =~ m/^($langdef->{scenario_outline})/
-                && !@{ $scenario->data } )
-            {
-                die parse_error_from_line(
-                    "Outline scenario expects 'Examples:' section", $line );
-            }
-
             if ( $type =~ m/^($langdef->{background})/ ) {
                 $feature->background($scenario);
             } else {
@@ -218,74 +325,111 @@ m/^((?:$langdef->{background})|(?:$langdef->{scenario})|(?:$langdef->{scenario_o
 
             # Scenario-level tags
         } elsif ( $line->content =~ m/^\s*\@\w/ ) {
-            my @tags = $line->content =~ m/\@([^\s]+)/g;
-            push( @scenario_tags, @tags );
+            push @tags, ( $line->content =~ m/\@([^\s]+)/g );
 
         } else {
             die parse_error_from_line( "Malformed scenario line", $line );
         }
     }
 
+    $self->_finish_scenario( $feature );
     return $feature, $self->_remove_next_blanks(@lines);
 }
 
+my $warned_mixed_comments = 0;
+
 sub _extract_steps {
     my ( $self, $feature, $scenario, @lines ) = @_;
 
     my $langdef   = $self->{langdef};
     my @givens    = split( /\|/, $langdef->{given} );
     my $last_verb = $givens[-1];
+    my $last_line_was_comment = 0;
 
-    while ( my $line = shift(@lines) ) {
-        next if $line->is_comment;
-        last if $line->is_blank;
 
-        # Conventional step?
-        if ( $line->content =~
-m/^((?:$langdef->{given})|(?:$langdef->{and})|(?:$langdef->{when})|(?:$langdef->{then})|(?:$langdef->{but})) (.+)/
-          )
-        {
-            my ( $verb, $text ) = ( $1, $2 );
-            my $original_verb = $verb;
-            $verb = 'Given' if $verb =~ m/^($langdef->{given})$/;
-            $verb = 'When'  if $verb =~ m/^($langdef->{when})$/;
-            $verb = 'Then'  if $verb =~ m/^($langdef->{then})$/;
-            $verb = $last_verb
-              if $verb =~ m/^($langdef->{and})$/
-              or $verb =~ m/^($langdef->{but}$)/;
-            $last_verb = $verb;
+    my ( $verb, $text );
+    while ( @lines and
+            ($lines[0]->is_comment
+             or ($verb, $text) = $self->_is_step_line( 1, $lines[0]->content ) ) ) {
+        my $line = shift @lines;
+        if ($line->is_comment) {
+            $last_line_was_comment = 1;
+            next;
+        }
 
-            my $step = Test::BDD::Cucumber::Model::Step->new(
-                {
-                    text          => $text,
-                    verb          => $verb,
-                    line          => $line,
-                    verb_original => $original_verb,
-                }
+        if ($last_line_was_comment) {
+            # don't issue this warning if the comment is after
+            warn 'Mixing comments and steps is deprecated: not allowed in Gherkin'
+                unless $warned_mixed_comments;
+            $warned_mixed_comments = 1;
+        }
+
+        my $original_verb = $verb;
+        $verb = 'Given' if $verb =~ m/^($langdef->{given})$/;
+        $verb = 'When'  if $verb =~ m/^($langdef->{when})$/;
+        $verb = 'Then'  if $verb =~ m/^($langdef->{then})$/;
+        $verb = $last_verb
+            if $verb =~ m/^($langdef->{and})$/
+            or $verb =~ m/^($langdef->{but}$)/;
+        $last_verb = $verb;
+
+        # Remove the ending space for languages that
+        # have it, for backward compatibility
+        $original_verb =~ s/ $//;
+        my $step = Test::BDD::Cucumber::Model::Step->new(
+            {
+                text          => $text,
+                verb          => $verb,
+                line          => $line,
+                verb_original => $original_verb,
+            }
             );
 
-            @lines =
-              $self->_extract_step_data( $feature, $scenario, $step, @lines );
+        @lines =
+            $self->_extract_step_data( $feature, $scenario, $step, @lines );
 
-            push( @{ $scenario->steps }, $step );
-
-            # Outline data block...
-        } elsif ( $line->content =~ m/^($langdef->{examples}):$/ ) {
-            return $self->_extract_table( 6, $scenario,
-                $self->_remove_next_blanks(@lines) );
-        } elsif ( $line->content =~
-            m/^(?:(?:$langdef->{scenario})|(?:$langdef->{scenario_outline})):/ )
-        {
-            # next scenario begins here
-            return ($line, @lines);
-        } else {
-            die parse_error_from_line( "Malformed step line", $line );
-        }
+        push( @{ $scenario->steps }, $step );
     }
 
     return $self->_remove_next_blanks(@lines);
 }
 
+
+sub _extract_examples_description {
+    my ( $self, $examples, @lines ) = @_;
+
+    while ( my $line = shift @lines ) {
+        next if $line->is_comment;
+
+        my $content = $line->content;
+        return ( $line, @lines )
+            if $self->_is_table_line( $content )
+               or $self->_is_examples_line( $content )
+               or $self->_is_tags_line( $content )
+               or $self->_is_scenario_line( $content );
+
+        push @{$examples->description}, $line;
+    }
+
+    return @lines;
+}
+
+sub _extract_scenario_description {
+    my ( $self, $scenario, @lines ) = @_;
+
+    while ( @lines
+            and ($lines[0]->is_comment
+                 or (not $self->_is_step_line(0, $lines[0]->content)
+                     and not $self->_is_examples_line($lines[0]->content)
+                     and not $self->_is_tags_line($lines[0]->content)
+                     and not $self->_is_scenario_line($lines[0]->content) ) )
+        ) {
+        push @{$scenario->description}, shift(@lines);
+    }
+
+    return @lines;
+}
+
 sub _extract_step_data {
     my ( $self, $feature, $scenario, $step, @lines ) = @_;
     return unless @lines;
